
require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/core'
require 'tableview'
require 'open3'

module TableTest

  class TestTable < Test::Unit::TestCase
    include Amrita2
    include Amrita2::Core
    include TableView

    Data1 = Struct.new(:a, :b, :c, :d, :e)

    TABLE_TEMPLATE = <<-END
    <table>
      <thead>
        <tr><th></th><th></th></tr>
      </thead>
      <tbody>
        <tr><th></th><td></td></tr>
      </tbody>
    </table>
    END

    def test_item
      v = ViewSetting.new {
        row(:a)
        column(:b)
        url do |d|
          "http://xxx.walrus-ruby.org/somedata/#{d.e}"
        end
        short_display do |d|
          "#{d.c}:#{d.d}"
        end
      }
      data = Data1.new(1, 2, 3, 4, 5)
      i = v.create_item(data)
      assert_equal(1, i.row)
      assert_equal(2, i.column)
      
      assert_equal('3:4', i.short_display)
      assert_equal('http://xxx.walrus-ruby.org/somedata/5', i.url)
    end

    def test_pagedata
      v = ViewSetting.new {
        row(:a)
        column(:b)
        url do |d|
          "http://xxx.walrus-ruby.org/somedata/#{d.e}"
        end
        short_display do |d|
          "#{d.c}:#{d.d}"
        end
      }

      data = []
      data << v.create_item(Data1.new(1,1,1,1,11))
      data << v.create_item(Data1.new(2,2,2,2,22))
      data << v.create_item(Data1.new(3,3,3,3,33))

      pd = PageData.new(data)
      assert_equal(3, pd.rows.size)
      assert_equal([1,2,3], pd.row_headers)
      assert_equal([1,2,3], pd.col_headers)
      
      item1 = pd[0,0][0]
      assert_equal(1, item1.data.a)
      assert_equal(11, item1.data.e)
      item1 = pd[0,1][0]
      assert_equal(nil, item1)
      item1 = pd[0,2][0]
      assert_equal(nil, item1)
      item2 = pd[1, 1][0]
      assert_equal(2, item2.data.a)
      assert_equal(22, item2.data.e)
      item3 = pd[2, 2][0]
      assert_equal(3, item3.data.a)
      assert_equal(33, item3.data.e)
    end

    def test_pagedata2
      v = ViewSetting.new {
        row(:a)
        column(:b)
        url do |d|
          "http://xxx.walrus-ruby.org/somedata/#{d.e}"
        end
        short_display do |d|
          "#{d.c}:#{d.d}"
        end
      }

      data = []
      (1..3).each do |n1|
        (1..3).each do |n2|
          (1..3).each do |n3|
            (1..3).each do |n4|
              data << v.create_item(Data1.new(n1, n2, n3, n4, n1*n2*n3*n4))
            end
          end
        end
      end

      pd = PageData.new(data)
      pd = PageData.new(data)
      assert_equal(3, pd.rows.size)
      assert_equal([1,2,3], pd.row_headers)
      assert_equal([1,2,3], pd.col_headers)
      
      item1 = pd[0,0][0]
      assert_equal(1, item1.data.a)
      assert_equal(1, item1.data.e)
      item1 = pd[0,1][0]
      assert_equal(1*2*1*1, item1.data.e)
      item1 = pd[0,1][1]
      assert_equal(1*2*2*1, item1.data.e)
      item1 = pd[0,2][0]
      assert_equal(1*3*1*1, item1.data.e)
      item1 = pd[0,2][2]
      assert_equal(1*3*3*1, item1.data.e)
      item2 = pd[1, 1][0]
      assert_equal(2, item2.data.a)
      assert_equal(2*2*1*1, item2.data.e)
      item22 = pd[1, 1][2]
      assert_equal(2, item22.data.a)
      assert_equal(2*2*3*1, item22.data.e)
      item3 = pd[2, 2][0]
      assert_equal(3, item3.data.a)
      assert_equal(3*3*1*1, item3.data.e)
      assert_equal(3*3*3*1, pd[2,2][2].data.e)
    end

    def test_tabledisp
      v = ViewSetting.new {
        row(:a)
        column(:b)
        url do |d|
          "http://xxx.walrus-ruby.org/somedata/#{d.e}"
        end
        short_display do |d|
          "#{d.c}:#{d.d}"
        end
      }

      data = []
      data << v.create_item(Data1.new(1,1,1,1,11))
      data << v.create_item(Data1.new(2,2,2,2,22))
      data << v.create_item(Data1.new(3,3,3,3,33))

      pd = PageData.new(data)
      
      doc = REXML::Document.new  TABLE_TEMPLATE
      t = TABLE_SPEC.create_template(doc)

      t.expand_template(out="") do |m|
        m.table do |mt| 
          mt.header do |mh|
            mh.first_col_header("")
            pd.col_headers.each do |ch|
              mh.col_header(ch)
            end
          end
          pd.row_headers.each_with_index do |row_header, row_index|
            mt.detail do |md|
              md.row_header(row_header)
              pd.col_headers.size.times do |i|
                d = pd[row_index, i].collect {|i| i.data.e}.join("/")
                md.data(d)
              end
            end
          end
        end
      end

      formated_out = ""
      Open3.popen3("w3m -T text/html") do |i, o, e|
        i.puts out
        i.close
        formated_out = o.read
      end

      assert_equal(<<END, formated_out)
  1  2  3
1 11
2    22
3       33

END
    end

    def test_tabledisp2
      v = ViewSetting.new {
        row(:a)
        column(:b)
        url do |d|
          "http://xxx.walrus-ruby.org/somedata/#{d.e}"
        end
        short_display do |d|
          "#{d.c}:#{d.d}"
        end
      }

      data = []
      data << v.create_item(Data1.new(1,1,1,1,11))
      data << v.create_item(Data1.new(1,1,1,1,12))
      data << v.create_item(Data1.new(1,1,1,1,13))
      data << v.create_item(Data1.new(2,2,2,2,22))
      data << v.create_item(Data1.new(3,3,3,3,33))
      data << v.create_item(Data1.new(4,3,2,1,4321))
      data << v.create_item(Data1.new(4,1,4,1,4141))
      data << v.create_item(Data1.new(5,5,5,5,51))
      data << v.create_item(Data1.new(5,5,5,5,52))

      pd = PageData.new(data)
      
      doc = REXML::Document.new  TABLE_TEMPLATE
      t = TableView::TABLE_SPEC.create_template(doc)

      t.expand_template(out="") do |m|
        m.table do |mt| 
          mt.header do |mh|
            mh.first_col_header("")
            pd.col_headers.each do |ch|
              mh.col_header(ch)
            end
          end
          pd.row_headers.each_with_index do |row_header, row_index|
            mt.detail do |md|
              md.row_header(row_header)
              pd.col_headers.size.times do |i|
                d = pd[row_index, i].collect {|i| i.data.e}.join("/")
                md.data(d)
              end
            end
          end
        end
      end

      t.expand_template(out2="") do |m|
        pd.format(m) do |items|
          items.collect {|i| i.data.e}.join("/")
        end
      end

      assert_equal(out, out2)

      formated_out = ""
      Open3.popen3("w3m -T text/html") do |i, o, e|
        i.puts out
        i.close
        formated_out = o.read
      end
      # puts formated_out
      assert_equal(<<END, formated_out)
      1     2   3     5
1 11/12/13
2           22
3              33
4 4141         4321
5                   51/52

END
    end
  end    
end

if __FILE__ == $0
  include TableTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestTable.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end

# ruby -I../../lib test_table.rb|ruby -pe '$_.sub!(/^ */, "") ' 
