#
# This script tests Ruby part and generate a test script for
# JavaScript parts to #{TEST_HTML_PATH}.
# Run this HTML file with testRunnne.html of
# JsUnit(http://www.edwardh.com/jsunit/) 

require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/template'
require 'amrita2/js'
require 'rexml/document'

# ruby -I~/amrita2/lib ~/amrita2/test/test_js.rb   |ruby -pe '$_.sub!(/^ */, "") ' 

JSUNIT_URL='/jsunit'
HOME=File::expand_path("~/amrita2")
TEST_HTML_PATH="#{HOME}/t.html"

TEMPLATE_TEXT = <<END
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />  
<title>JsUnit Assertion Tests</title>
<link rel="stylesheet" type="text/css" href="#{JSUNIT_URL}/css/jsUnitStyle.css" />
<script language="JavaScript" type="text/javascript" src="#{JSUNIT_URL}/app/jsUnitCore.js"></script>
<script id='runtime' language="JavaScript" type="text/javascript"></script>
<script id='script' language="JavaScript" type="text/javascript"></script>
</head>

<body>
<h1>Amrita Tests</h1>

<p>test test</p>
</body>
</html>
END

$T = "\n\n"

module Amrita2JSTest
  include Amrita2
  include Amrita2::JS

  def self.generate_test_html
    t = TemplateText.new(TEMPLATE_TEXT)
    jssrc = Amrita2::JS::RUNTIME_SRC + $T + <<-END

    function assertHTML(ans, res) { 
        var e = document.createElement('x') ;
        e.appendChild(res) ;
        assertEquals(ans, e.innerHTML);
    }

    END

    File::open(TEST_HTML_PATH, 'w') do |f|
      t.expand(f, :script => SanitizedString[jssrc])
    end
  end

  class TestCodeGenerator < Test::Unit::TestCase
    def setup
      @js = JS::CodeGenerator.new
    end

    def teardown
      $T << @js.results.join(NL)
      $T << NL
    end

    def test_function
      @js.define_function(:test_function) do 
        @js.code %[debug("aaaa");]
        @js.code %[assert(true);]
      end
    end

    def test_if
      @js.define_function(:test_if) do 
        @js.code %[]
        @js.code %[var t = 1; ]
        @js.if_("1 == 1") do
          @js.code %[t = 2]
        end
        @js.code %[assertEquals(2, t);]
        @js.if_("1 == 0") do
          @js.code %[assert(false);]
          @js.code %[t = 3]
        end
        @js.code %[assertEquals(2, t);]
        @js.if_("1 == 0") do
          @js.code %[assert(false);]
          @js.else_ do 
            @js.code %[t = 3]
          end
        end
        @js.code %[assertEquals(3, t);]
      end
    end

    def test_dom
      @js.define_function(:test_dom) do 
        @js.code <<-END
        var output = document.getElementsByTagName('p')[0];
        var br = document.createElement("br") ;
        output.appendChild(br);
        var text = document.createTextNode('Hello, world.');
        var em = document.createElement("em") ;
        em.appendChild(text)
        output.appendChild(em);

        var c = output.childNodes ;
        assertEquals(3, c.length) ;

        assertEquals("test test<BR><EM>Hello, world.</EM>", output.innerHTML)  ;
        output.removeChild(em) ;
        output.removeChild(br) ;
        assertEquals("test test", output.innerHTML)  ;
        END
      end
    end

    def test_element
      doc = REXML::Document.new  "<a href='http://amrita2.rubyforge.org/'>Amrita2</a>"
      e = doc.root 
      @js.define_function(:test_element) do 
        @js.code("var e =") ; @js.element(e) 
        @js.code("var e2 = document.createElement('x') ;")
        @js.code %[e2.appendChild(e) ;]
        @js.code %[assertEquals('<A href="http://amrita2.rubyforge.org/">Amrita2</A>', e2.innerHTML);]
      end
    end

    def test_element2
      doc = REXML::Document.new  "<a href='http://amrita2.rubyforge.org/'>Amrita2</a>"
      e = doc.root 
      @js.define_function(:test_element2) do 
        @js.code("var e =") ; @js.element(e) { 'foo' }
        @js.code("var e2 = document.createElement('x') ;")
        @js.code %[e2.appendChild(e) ;]
        @js.code %[assertEquals('<A href="http://amrita2.rubyforge.org/">foo</A>', e2.innerHTML);]
      end
    end

    def test_element3
      doc = REXML::Document.new  "<h1>x<a href='http://amrita2.rubyforge.org/'>Amrita2</a>y</h1>"
      e = doc.root 
      @js.define_function(:test_element3) do 
        @js.code("var e =") ; @js.element(e)
        @js.code("var e2 = document.createElement('x') ;")
        @js.code %[e2.appendChild(e) ;]
        @js.code %[assertEquals('<H1>x<A href="http://amrita2.rubyforge.org/">Amrita2</A>y</H1>', e2.innerHTML);]
      end
    end
  end

  class TestJson < Test::Unit::TestCase
    def setup
      @js = JS::CodeGenerator.new
    end

    def teardown
      $T << @js.results.join(NL)
      $T << NL
    end

    def test_json
      @js.define_function(:test_json) do 
        @js.code <<-END
        var a = #{1.to_json} ;
        assertEquals(1, a) ;
        a = #{"aaa".to_json} ;
        assertEquals("aaa", a) ;
        END
      end
    end

    def test_json_array
      @js.define_function(:test_json_array) do 
        @js.code <<-END
        var a = #{[1, 2, 3].to_json} ;
        assertEquals(1, a[0]) ;
        assertEquals(2, a[1]) ;
        assertEquals(3, a[2]) ;
        END
      end
    end

    def test_json_hash
      @js.define_function(:test_hash) do 
        @js.code <<-END
        a = #{{ :xxx=>'yyy', 'abc' => 'efg'}.to_json} ;
        assertEquals('yyy', a['xxx']) ;
        assertEquals('yyy', a.xxx) ;
        assertEquals('efg', a['abc']) ;
        assertEquals('efg', a.abc) ;
        END
      end
    end
  end

  class TestRuntime < Test::Unit::TestCase
    def setup
      @js = JS::CodeGenerator.new
    end

    def teardown
      $T << @js.results.join(NL)
      $T << NL
    end

    def test_createtext
      @js.define_function(:test_createtext) do 
        @js.code <<-END
        var e = document.createElement('x') ;
        amrita_appendText(e, 'aaa') ;
        assertEquals('aaa', e.innerHTML) ;
        amrita_appendText(e, 'bbb') ;
        assertEquals('aaabbb', e.innerHTML) ;
        END
      end
    end

    def test_createelement
      @js.define_function(:test_createelement) do 
        @js.code <<-END
        var e = document.createElement('x') ;
        amrita_appendElement(e, 'aaa', { }) ;
        assertEquals('<aaa></aaa>', e.innerHTML) ;
        amrita_appendElement(e, 'aaa', { 'abc': 'efg'}) ;
        assertEquals('<aaa></aaa><aaa abc="efg"></aaa>', e.innerHTML) ;
        e = document.createElement('x') ;
        amrita_appendElement(e, 'aaa', { 'abc': 'efg'}, 'text') ;
        assertEquals('<aaa abc="efg">text</aaa>', e.innerHTML) ;
        e = document.createElement('x') ;
        amrita_appendElement(e, 'aaa', { 'abc': 'efg'}, function() { return 'func'}) ;
        assertEquals('<aaa abc="efg">func</aaa>', e.innerHTML) ;

        assertHTML('<x><aaa abc="efg">func</aaa></x>', e)
        END
      end
    end

    def test_movechildren
      @js.define_function(:test_movechidlren) do 
        @js.code <<-END
        var e = document.createElement('x') ;
        amrita_appendElement(e, 'aaa', { }) ;
        assertEquals('<aaa></aaa>', e.innerHTML) ;
        amrita_appendText(e, 'xxx')
        amrita_appendElement(e, 'aaa', { 'abc': 'efg'}) ;
        assertEquals('<aaa></aaa>xxx<aaa abc="efg"></aaa>', e.innerHTML) ;

        var to = document.createElement('x') ;
        amrita_moveChildren(e, to) ;
        assertEquals('<aaa></aaa>xxx<aaa abc="efg"></aaa>', to.innerHTML) ;
        END
      end
    end
  end

  class TestTemplate < Test::Unit::TestCase
    def setup
      @js = JS::CodeGenerator.new
    end

    def teardown
      $T << @js.results.join(NL)
      $T << NL
    end

    def test_simple
      t = TemplateText.new '<x><em id="aaa">xxx</em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa)
        end
      end
      t.expand(out="") do |m|
        m.aaa('123')
      end
      assert_equal("<x><em id='aaa'>123</em></x>", out)

      @js.define_function(:test_tmpl_simple) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, function(tt) { tt.aaa('123') ; }) ;
        assertHTML("<outer><x><EM id=aaa>123</EM></x></outer>", a) ;
        END
      end
    end

    def test_simple2
      t = TemplateText.new '<x><em id="aaa">xxx</em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa)
        end
      end
      t.expand(out="") do |m|
        m.aaa('123')
        m.aaa('456')
      end
      assert_equal("<x><em id='aaa'>123</em><em id='aaa'>456</em></x>", out)

      @js.define_function(:test_tmpl_simple2) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, function(tt) { tt.aaa('123') ;tt.aaa('456') ; }) ;
        assertHTML("<outer><x><EM id=aaa>123</EM><EM id=aaa>456</EM></x></outer>", a) ;
        END
      end
    end
    def test_simple3
      t = TemplateText.new '<x><em id="aaa">xxx</em><a id="bbb">xxx</a></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa)
          dynamic_element(:bbb, :replace_attr=>true)
        end
      end
      t.expand(out="") do |m|
        m.aaa('123')
        m.bbb('456', :href=>'http://amrita2.rubyforge.org/')
      end
      assert_equal("<x><em id='aaa'>123</em><a href='http://amrita2.rubyforge.org/' id='bbb'>456</a></x>", out)

      @js.define_function(:test_tmpl_simple3) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, function(t) { t.aaa('123') ; t.bbb('456', { 'href': 'http://amrita2.rubyforge.org/'})}) ;
        assertHTML('<outer><x><EM id=aaa>123</EM><A id=bbb href="http://amrita2.rubyforge.org/">456</A></x></outer>', a) ;
        END
      end
    end

    def test_nest
      t = TemplateText.new '<x><em id="aaa"><code id="bbb">xxx</code></em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa) do
            dynamic_element(:bbb)
          end
        end
      end
      t.expand(out="") do |m|
        m.aaa do |mm|
          mm.bbb('123')
        end
      end
      assert_equal("<x><em id='aaa'><code id='bbb'>123</code></em></x>", out)

      @js.define_function(:test_tmpl_nest) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, function(t) { debug(555); t.aaa(function(t){ debug(666) ;t.bbb('123') }) ; }) ;
        assertHTML("<outer><x><EM id=aaa><CODE id=bbb>123</CODE></EM></x></outer>", a) ;
        END
      end
    end

    def test_hello
      t = TemplateText.new <<-END
<div>
<h1 id='title'>title will be inserted here</h1>
<p id='body'>body text will be inserted here</p>
<ul id='list'><li id='item' /></ul>
<p id='active'>For detail see<a id='link'></a></p>
</div>
END

      @js.define_function(:test_tmpl_hello) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var data = { 
          'title': 'Hello JavaScript', 
          'body': 'BODY',
          'list': { 'item': [1,2,3] },
          'active': function(t) {
             t.link('Amrita2 Hope Page', { 'href': 'http://amrita2.rubyforge.org/'} ) ;
           }
        } ;

        var a = tmpl.expand(outer, data) ;
        debug(a.innerHTML) ;
        END
        #@js.code %q[assertHTML('<outer>\n<DIV>\n<H1 id=title>Hello JavaScript</H1>\n<P id=body>BODY</P>\n<UL id=list>\n<LI id=item>1</LI>\n<LI id=item>2</LI>\n<LI id=item>3</LI></UL>\n<P id=active>For detail see<A id=link href=\"http://amrita2.rubyforge.org/\">Amrita2 Hope Page</A></P>\n</DIV></outer>', a) ;]
      end
    end
  end

  class TestExpander < Test::Unit::TestCase
    def setup
      @js = JS::CodeGenerator.new
    end

    def teardown
      $T << @js.results.join(NL)
      $T << NL
    end

    def test_simple
      t = TemplateText.new '<x><em id="aaa">xxx</em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa)
        end
      end
      t.expand(out="", :aaa=>'123') 
      assert_equal("<x><em id='aaa'>123</em></x>", out)

      @js.define_function(:test_auto_simple) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, { 'aaa': '12' }) ;
        assertHTML("<outer><x><EM id=aaa>12</EM></x></outer>", a) ;
        END
      end
    end

    def test_simple_array
      t = TemplateText.new '<x><em id="aaa">xxx</em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa)
        end
      end
      t.expand(out="", :aaa=>'123') 
      assert_equal("<x><em id='aaa'>123</em></x>", out)

      @js.define_function(:test_auto_array) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, { 'aaa': [1,2,3] }) ;
        assertHTML("<outer><x><EM id=aaa>1</EM><EM id=aaa>2</EM><EM id=aaa>3</EM></x></outer>", a) ;
        END
      end
    end

    def test_nest
      t = TemplateText.new '<x><em id="aaa"><code id="bbb">xxx</code></em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa) do
            dynamic_element(:bbb)
          end
        end
      end
      
      @js.define_function(:test_auto_nest) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, { 'aaa' : { 'bbb': 333 }}) ;
        assertHTML("<outer><x><EM id=aaa><CODE id=bbb>333</CODE></EM></x></outer>", a) ;
        END
      end
    end

    def test_nest_array
      t = TemplateText.new '<x><em id="aaa"><code id="bbb">xxx</code></em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa) do
            dynamic_element(:bbb)
          end
        end
      end
      
      @js.define_function(:test_auto_nestaray) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, { 'aaa' : { 'bbb': [1,2] }}) ;
        assertHTML("<outer><x><EM id=aaa><CODE id=bbb>1</CODE><CODE id=bbb>2</CODE></EM></x></outer>", a) ;
        END
      end
    end

    def test_nest_array2
      t = TemplateText.new '<x><em id="aaa"><code id="bbb">xxx</code></em></x>' do 
        Amrita2::define_template_spec do 
          dynamic_element(:aaa) do
            dynamic_element(:bbb)
          end
        end
      end
      
      @js.define_function(:test_auto_nestaray2) do 
        t.js_template('t', @js)
        @js.code <<-END
        var tmpl = new t() ; 
        var outer = document.createElement('outer') ; 
        var a = tmpl.expand(outer, { 'aaa' : [ { 'bbb': [1,2] }, { 'bbb': [3,4]}] }) ;
        assertHTML("<outer><x><EM id=aaa><CODE id=bbb>1</CODE><CODE id=bbb>2</CODE></EM><EM id=aaa><CODE id=bbb>3</CODE><CODE id=bbb>4</CODE></EM></x></outer>", a) ;
        END
      end
    end
  end

   class TestParts < Test::Unit::TestCase
     def setup
       @js = JS::CodeGenerator.new
     end

     def teardown
       $T << @js.results.join(NL)
       $T << NL
     end

     def test_simple
       t = TemplateText.new '<x><span id="p1"></span></x>' do 
         Amrita2::define_template_spec do 
           parts_element(:p1, :place_holder=>true) 
         end
       end
       t.setup
       assert_equal("<span id='p1'>123</span>", t.p1('123'))
       t.expand(out="") do
       end
       assert_equal("<x><span id='p1'></span></x>", out)
       @js.define_function(:test_parts_simple) do 
         t.js_template('t', @js)
         @js.code <<-END
         var tmpl = new t() ; 
         debug(1) ;
         var a = tmpl.p1('12') ;
         debug(4) ;
         assertHTML("<SPAN id=p1>12</SPAN>", a) ;
         END
       end
     end

     def test_nest
       t = TemplateText.new '<x><span id="p1"><em id="aaa">xxx</em></span></x>' do 
         Amrita2::define_template_spec do 
           parts_element(:p1, :place_holder=>true) do 
             dynamic_element(:aaa)
           end
         end
       end
       t.setup
       assert_equal("<span id='p1'><em id='aaa'>123</em></span>", t.p1(:aaa=>'123'))
       t.expand(out="") do
       end
       assert_equal("<x><span id='p1'></span></x>", out)
       @js.define_function(:test_parts_nest) do 
         t.js_template('t', @js)
         @js.code <<-END
         var tmpl = new t() ; 
         debug(1) ;
         var a = tmpl.p1({ 'aaa': '12' }) ;
         debug(4) ;
         assertHTML("<SPAN id=p1><EM id=aaa>12</EM></SPAN>", a) ;

         var a = tmpl.p1(function(t) { t.aaa('12'); }) ;
         debug(5) ;
         assertHTML("<SPAN id=p1><EM id=aaa>12</EM></SPAN>", a) ;

         var a = tmpl.p1({ 'aaa': [1,2,3] }) ;
         debug(6) ;
         assertHTML("<SPAN id=p1><EM id=aaa>1</EM><EM id=aaa>2</EM><EM id=aaa>3</EM></SPAN>", a) ;
         END
       end
     end
   end
end

if __FILE__ == $0
  include Amrita2JSTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestParts.suite
    suite << TestExpander.suite
    suite << TestRuntime.suite
    suite << TestTemplate.suite
    suite << TestJson.suite
    suite << TestCodeGenerator.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
  Amrita2JSTest::generate_test_html
end
